@@ -28,7 +28,7 @@
 /* Default to the English language. */
 static const char* in_language = "en";
 
-/* Dicationary can be and should be shared by all. */
+/* Ditionary can be and should be shared by all. */
 static Dictionary dict = NULL;
 #ifdef HAVE_STDATOMIC_H
 static atomic_flag dict_is_init = ATOMIC_FLAG_INIT;
@@ -49,7 +49,7 @@ static void setup_panic_parse_options(Parse_Options opts)
 	parse_options_set_repeatable_rand(opts, false);
 	parse_options_set_disjunct_cost(opts, 3.7f);
 	parse_options_set_min_null_count(opts, 1);
-	parse_options_set_max_null_count(opts, MAX_SENTENCE);
+	parse_options_set_max_null_count(opts, 250);
 	parse_options_set_max_parse_time(opts, 60);
 	parse_options_set_islands_ok(opts, true);
 	parse_options_set_short_length(opts, 6);
@@ -60,7 +60,7 @@ static void setup_panic_parse_options(Parse_Options opts)
 	parse_options_set_display_morphology(opts, true);
 }
 
-static inline void test(void)
+static inline void do_test(void)
 {
 #ifdef DEBUG
 	printf("%d\n", word_contains("said", PAST_TENSE_FORM_MARKER, dict));
@@ -124,7 +124,7 @@ static void global_init(JNIEnv *env)
 
 	dict = dictionary_create_lang(in_language);
 	if (!dict) throwException(env, "Error: unable to open dictionary");
-	else test();
+	else do_test();
 
 	dict_version = linkgrammar_get_dict_version(dict);
 	prt_error("Info: JNI: dictionary language '%s' version %s\n",
