@@ -288,6 +288,10 @@ class LG_Error(Exception):
     #def format(lgerror):
     #    return clg.lg_error_formatmsg(lgerror)
 
+    @staticmethod
+    def flush():
+        return clg.lg_error_flush()
+
     @staticmethod
     def printall(ehandler_func, ehandler_data=None):
         return clg._py_error_printall((ehandler_func, ehandler_data))
@@ -434,6 +438,18 @@ class Linkage(object):
     def constituent_tree(self, mode=1):
         return clg.linkage_print_constituent_tree(self._obj, mode)
 
+    def word_byte_start(self, w):
+        return clg.linkage_get_word_byte_start(self._obj, w)
+
+    def word_byte_end(self, w):
+        return clg.linkage_get_word_byte_end(self._obj, w)
+
+    def word_char_start(self, w):
+        return clg.linkage_get_word_char_start(self._obj, w)
+
+    def word_char_end(self, w):
+        return clg.linkage_get_word_char_end(self._obj, w)
+
 
 class LG_TimerExhausted(LG_Error):
     pass
