@@ -33,7 +33,6 @@
 
 #include "error.h"
 #include "lg_assert.h"
-#include "wcwidth.h"
 
 #ifdef HAVE_ALLOCA_H
 # include <alloca.h>
@@ -112,7 +111,12 @@ void *alloca (size_t);
 #endif
 /* Note that "#define _CRT_RAND_S" is needed before "#include <stdlib.h>" */
 #define rand_r(seedp) rand_s(seedp)
+
+/* No strtok_s in these versions and their strtok_r is incompatible. */
+#if _WINVER != 0x501 /* XP */ && _WINVER != 0x502 /* Server 2003 */
 #define strtok_r strtok_s
+#define HAVE_STRTOK_R
+#endif
 
 /* Native windows has locale_t, and hence HAVE_LOCALE_T is defined here.
  * However, MinGW currently doesn't have locale_t. If/when it has locale_t,
@@ -250,12 +254,6 @@ typedef int locale_t;
 #define GNUC_UNUSED
 #endif
 
-/**
- * Return the width, in text-column-widths, of the utf8-encoded
- * string.  This is needed when printing formatted strings.
- */
-size_t utf8_strwidth(const char *);
-
 /**
  * Return the length, in codepoints/glyphs, of the utf8-encoded
  * string.  The string is assumed to be null-terminated.
@@ -447,17 +445,18 @@ char *safe_strdup(const char *u);
 /* Simple, cheap, easy dynamic string. */
 typedef struct
 {
-  char *str;
-  size_t end;
-  size_t len;
+	char *str;
+	size_t end;
+	size_t len;
 } dyn_str;
 
 dyn_str* dyn_str_new(void);
 void dyn_str_delete(dyn_str*);
 void dyn_strcat(dyn_str*, const char*);
+char * dyn_str_take(dyn_str*);
+const char * dyn_str_value(dyn_str*);
 
 size_t altlen(const char **);
-void altappend(Sentence, const char ***, const char *);
 
 /* routines for allocating basic objects */
 void init_memusage(void);
@@ -478,15 +477,6 @@ size_t get_max_space_used(void);
 
 
 char * get_default_locale(void);
-char * join_path(const char * prefix, const char * suffix);
-
-FILE * dictopen(const char *filename, const char *how);
-void * object_open(const char *filename,
-                   void * (*opencb)(const char *, const void *),
-                   const void * user_data);
-
-bool file_exists(const char * dict_name);
-char * get_file_contents(const char *filename);
 void set_utf8_program_locale(void);
 bool try_locale(const char *);
 
@@ -495,9 +485,9 @@ bool try_locale(const char *);
  */
 static inline unsigned int next_power_of_two_up(unsigned int i)
 {
-   unsigned int j=1;
-   while (j<i) j <<= 1;
-   return j;
+	unsigned int j=1;
+	while (j<i) j <<= 1;
+	return j;
 }
 
 #endif /* _LINK_GRAMMAR_UTILITIES_H_ */
