@@ -223,6 +223,7 @@ class Link : public Atom
 		void add_to_incoming_set(AtomType);
 		void remove_from_incoming_set(AtomType);
 		Link* append(Atom*) const;
+		Link* replace(Atom*, Atom*) const;
 
 		virtual bool operator==(const Atom*) const;
 		virtual Link* clone() const { return new Link(*this); }
@@ -232,7 +233,7 @@ class Link : public Atom
 };
 
 
-// An unhygenic for-each loop, to simplify iterating over
+// An unhygienic for-each loop, to simplify iterating over
 // the outgoing set.  I don't see a more elegant way to do this,
 // just right now...
 // Anyway, this implements the semantics "foreach VAR of TYPENAME in LNK"
